unit Helios_ROQ_Engine;

{$I Global_Conditionals.INC}

interface

uses qne_RoQ;

Function IsRoqActive : Boolean; overload
Function IsRoqActive(Cinematic : PCinematic) : Boolean; overload

Function LoadRoq(Filename : String; RenderToScreen : Boolean) : integer;
Procedure ReadNextRoqFrame(Var Cinematic : PCinematic);
Procedure RenderRoq(Var Cinematic : PCinematic);
Procedure RenderROQs;
Procedure StopRoq(Var Cinematic : PCinematic);

Procedure DestroyRTSRoqs;
Procedure DestroyRoqs;

procedure Roq_MixAudio(Cinematic : PCinematic);
Function IsFullScreenRoqActive : Boolean;

Function LoadRoq_M(Filename : String; Loop,RenderToScreen,KeepOpen : Boolean; State : Integer) : Integer;

Procedure CloseRoq(var cin: PCinematic);

Const
  AUDIO_BUFFER_SAMPLE_COUNT = 65536;

Type
TCinematics = record
Cinematic : Array Of PCinematic;
Cinematic_No : Integer;
end;

Var
Cinematics : TCinematics;

AudioOffset: Cardinal;
AudioChannels: Word;
AudioBuffer: array[0..AUDIO_BUFFER_SAMPLE_COUNT-1] of SmallInt;

FullScreenROQPlaying : Boolean;

implementation

uses Math3d,OpenGL15,Helios_Timer,SysUtils{$IfDEF HELIOS_SOUND},Fmod,Fmodtypes{$EndIf}
     {$IfDEF HELIOS_APP}Helios_Global,Helios_Engine, Helios_FileSystem{$Else},Helios_Basic{$EndIf};

Function IsRoqActive : Boolean;
begin

If Cinematics.Cinematic_No > 0 then
Result := True
else
Result := False;

end;

Function IsRoqActive(Cinematic : PCinematic) : Boolean;
begin

Result := Assigned(Cinematic);

end;

Function IsFullScreenRoqActive : Boolean;
var
x : integer;
begin

Result := False;

for x := 0 to Cinematics.Cinematic_No-1 do
if IsRoqActive(Cinematics.Cinematic[x]) then
if Cinematics.Cinematic[x].RenderToScreen then
Result := True;

end;

Function AddRoqToList : Integer;
begin

Result := Cinematics.Cinematic_No;

inc(Cinematics.Cinematic_No);
SetLength(Cinematics.Cinematic,Cinematics.Cinematic_No);
Cinematics.Cinematic[Result] := nil;
end;

Function LoadRoq(Filename : String; RenderToScreen : Boolean) : integer;
var
dest : Array of Byte;
SID : Integer;
CID : Integer;
X : integer;
begin

// Does is it open?
if Cinematics.Cinematic_No > 0 then
for x := 0 to Cinematics.Cinematic_No-1 do
if IsRoqActive(Cinematics.Cinematic[x]) then
if Cinematics.Cinematic[x].filename = filename then
begin
Result := x;
exit;
end;

  try
  If RenderToScreen then
  DestroyRTSRoqs; // Destroys any current ROQ's that are RenderToScreens

  CID := AddRoqToList;
  Result := CID;

    Cinematics.Cinematic[CID] := RoQ_Open(Filename, False, AUDIO_BUFFER_SAMPLE_COUNT);
    if not Assigned(Cinematics.Cinematic[CID]) then
    begin
//      AddToConsole('An error has occured while opening RoQ file ' + Filename);
//      LockStream := False;
      Exit;
    end;

    Cinematics.Cinematic[CID].RenderToScreen := RenderToScreen;
    Cinematics.Cinematic[CID].StartTime := 0;
    Cinematics.Cinematic[CID].Loop := False;

//    LblInfo.Caption := Format('%dx%d, %d FPS',
//                              [Cinematic.Width, Cinematic.Height, Cinematic.FPS]);

    // Create a texture we can render to
    glGenTextures(1, @Cinematics.Cinematic[CID].Texture);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, Cinematics.Cinematic[CID].Texture);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

    SetLength(dest, Cinematics.Cinematic[CID].Width*Cinematics.Cinematic[CID].Height*3);

    If TextureQuality = 0 then
    begin
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Cinematics.Cinematic[CID].Width, Cinematics.Cinematic[CID].Height, 0, GL_RGB, GL_BYTE, dest);
    end
    else
    begin
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    gluBuild2DMipmaps(GL_TEXTURE_2D, 4, Cinematics.Cinematic[CID].Width, Cinematics.Cinematic[CID].Height, GL_RGB, GL_BYTE, dest);
    end;

    ReadNextRoqFrame(Cinematics.Cinematic[CID]);
//    AudioOffset := 0;
{    fmt.freq := 22050;
    fmt.format := AUDIO_S16;
    fmt.channels := 2;
    fmt.samples := SDL_BUFFER_SAMPLE_COUNT;
    fmt.callback := sdl_mixaudio;
    fmt.userdata := Cinematic;

    SDL_CloseAudio;
    SDL_OpenAudio(@fmt, nil);
    SDL_PauseAudio(0);  }
  finally
  //  LockStream := False;
  end;
end;

Procedure ReadNextRoqFrame(Var Cinematic : PCinematic);
begin
if not RoQ_ReadNextFrame(Cinematic) then
  begin
  RoQ_Restart(Cinematic);
  ReadNextRoqFrame(Cinematic);

  if Cinematic.Loop then
  begin
  Cinematic.State := 1;
  exit;
  end;

{$IfDEF HELIOS_SOUND}
 If Cinematic.FmodEmitterID > -1 then
 Sound.StopSound(Cinematic.FmodEmitterID);
{$EndIf}
 If not Cinematic.KeepOpen then
 If Cinematic.RenderToScreen then
 DestroyRTSRoqs
 else
 RoQ_Close(Cinematic);
 IsGame;
 //   SDL_CloseAudio;
  end
  else
  begin
  AudioChannels := Cinematic.AudioChannels;
//  FillChar(Cinematic.AudioBuffer[0], Cinematic.AudioSampleOffset * SizeOf(SmallInt), 0);
 // FillChar(AudioBuffer[0], AUDIO_BUFFER_SAMPLE_COUNT * SizeOf(SmallInt), 0);

 //   Move(Cinematic.AudioBuffer^, AudioBuffer[0], Cinematic.AudioSampleOffset * SizeOf(SmallInt));
  end;
end;

Procedure RenderRoq(Var Cinematic : PCinematic);
var
width,height,x,y,c,cur_frame : Integer;
src: PByteArray;
dest : Array of Byte;
begin
    if Pause then exit;

    If Cinematic.State = 1 then // If playing
    Cinematic.StartTime := Cinematic.StartTime + gTimer.FrameTime;
    cur_frame := Trunc((Cinematic.StartTime) * Cinematic.FPS);

    {$IfDEF HELIOS_SOUND}
    Roq_MixAudio(Cinematic);

    If AudioChannels > 0 then
    If Cinematic.FmodEmitterID >-1 then
    begin

    If Cinematic.State = 1 then
    If Cinematic.AudioSampleOffset > 0 then
    begin
//    Cinematic.AudioSampleOffset := 0;
//    Sound.StopSound(Cinematic.FmodEmitterID);
{    Sound.PlaySound(Cinematic.FmodEmitterID);
    Sound.Update(Camera.GetPositionWithHeight);}
    end;

   { If Cinematic.State = 1 then
    if not FSOUND_IsPlaying(Cinematic.FmodEmitterID) then
    Sound.PlaySound(Cinematic.FmodEmitterID);  }

    If Cinematic.State = 0 then
    if FSOUND_IsPlaying(Cinematic.FmodEmitterID) then
    Sound.StopSound(Cinematic.FmodEmitterID);

    end;
    {$EndIf}

    if cur_frame <= Cinematic.CurrentFrame-1 then
    Exit;

    Width := Cinematic.Width;
    Height := Cinematic.Height;
    SetLength(dest, Width*Height*3);

    c := 0;
    for y := 0 to Height-1 do
    begin
      src := @Cinematic.FrameBuffer[0, (Height-y-1) * Width * 3];

      for x := 0 to Width-1 do
      begin
        dest[c*3+0] := src[x*3+0] div 2;
        dest[c*3+1] := src[x*3+1] div 2;
        dest[c*3+2] := src[x*3+2] div 2;

        inc(c);
      end;
    end;

    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,Cinematic.Texture);

    If LastTexQuality <> TextureQuality then
    If TextureQuality = 0 then
    begin
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    end
    else
    begin
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    end;

    If TextureQuality = 0 then
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, Width, Height, 0, GL_RGB, GL_BYTE, dest)
    else
    gluBuild2DMipmaps(GL_TEXTURE_2D, 4, Width, Height, GL_RGB, GL_BYTE, dest);

    If Cinematic.State = 1 then
    ReadNextRoqFrame(Cinematic);
end;

Procedure RenderROQs;
var
x : integer;
begin

If Cinematics.Cinematic_No < 1 then exit;

for x := 0 to Cinematics.Cinematic_No-1 do
If IsRoqActive(Cinematics.Cinematic[x]) then
begin
RenderRoq(Cinematics.Cinematic[x]);

If x > Cinematics.Cinematic_No-1 then
exit;

If IsRoqActive(Cinematics.Cinematic[x]) then
If Cinematics.Cinematic[x].RenderToScreen then
begin
DrawGfxToScreen(Cinematics.Cinematic[x].Texture,0,0,SCREEN_WIDTH,SCREEN_HEIGHT,1,SetVector(1,1,1));
FullScreenROQPlaying := True;
end;
end;
end;

Procedure StopRoq(Var Cinematic : PCinematic);
begin

RoQ_Restart(Cinematic);
ReadNextRoqFrame(Cinematic);

end;

procedure Roq_MixAudio(Cinematic : PCinematic);
var
  samples: TSmallIntArray;
Begin
{$IfDEF HELIOS_SOUND}
  if AudioChannels = 0 then Exit;

  If Cinematic.FmodEmitterID = -1 then exit;

  If Cinematic.NewSound then
  if AudioChannels = 2 then
  begin
//  Move(Cinematic.AudioBuffer^, AudioBuffer[0], Cinematic.AudioSampleOffset * SizeOf(SmallInt));
    Sound.StopSound(Cinematic.FmodEmitterID);
    FSOUND_Sample_Upload(Sound.FEmitters[Cinematic.FmodEmitterID].SamplePtr,Cinematic.AudioBuffer,FSOUND_SIGNED);
    AudioOffset := Cinematic.AudioSampleOffset;
    Sound.PlaySound(Cinematic.FmodEmitterID);
    FillChar(Cinematic.AudioBuffer[0], Cinematic.AudioSampleOffset * SizeOf(SmallInt), 0);
  end
  else
  if AudioChannels = 1 then
  begin
//  Move(Cinematic.AudioBuffer^, AudioBuffer[0], Cinematic.AudioSampleOffset * SizeOf(SmallInt));
    FSOUND_Sample_Upload(Sound.FEmitters[Cinematic.FmodEmitterID].SamplePtr,Cinematic.AudioBuffer,FSOUND_SIGNED);
    AudioOffset := Cinematic.AudioSampleOffset;
    FillChar(Cinematic.AudioBuffer[0], Cinematic.AudioSampleOffset * SizeOf(SmallInt), 0);
  end;
  Cinematic.NewSound := False;
{$EndIf}
End;

Procedure DestroyRTSRoqs;
var
x : integer;
begin

if Cinematics.Cinematic_No < 1 then exit;

for x := 0 to Cinematics.Cinematic_No-1 do
If IsRoqActive(Cinematics.Cinematic[x]) then
If Cinematics.Cinematic[x].RenderToScreen then
begin
{$IfDEF HELIOS_SOUND}
If Cinematics.Cinematic[x].FmodEmitterID > -1 then
Sound.StopSound(Cinematics.Cinematic[x].FmodEmitterID);
{$EndIf}

RoQ_Close(Cinematics.Cinematic[x]);

If x+1 = Cinematics.Cinematic_No then // We can safely remove the last Roq in the list but we can't change the order of other active roq's
begin
Dec(Cinematics.Cinematic_No);
SetLength(Cinematics.Cinematic,Cinematics.Cinematic_No);
end;       

end;

IsGame;

end;

Procedure DestroyRoqs;
var
x : integer;
begin

if Cinematics.Cinematic_No > 0 then // we don't need to destroy if there is none
for x := 0 to Cinematics.Cinematic_No-1 do
If IsRoqActive(Cinematics.Cinematic[x]) then
begin
{$IfDEF HELIOS_SOUND}
If Cinematics.Cinematic[x].FmodEmitterID > -1 then
Sound.StopSound(Cinematics.Cinematic[x].FmodEmitterID);
{$EndIf}
RoQ_Close(Cinematics.Cinematic[x]);
end;

Cinematics.Cinematic_No := 0;
SetLength(Cinematics.Cinematic,Cinematics.Cinematic_No);

end;

Procedure CloseRoq(var cin: PCinematic);
begin
RoQ_Close(cin);
end;

Function LoadRoq_M(Filename : String; Loop,RenderToScreen,KeepOpen : Boolean; State : Integer) : Integer;
var
FN : String;
CinimaticID : integer;
begin
CinimaticID := -1;
Result := -1;

FN := DoesFileExist(Filename);
If FN = '' then
FN := DoesFileExist(Filename + '.roq');
If FN = '' then
FN := DoesFileExist('\video\' + Filename);
If FN = '' then
FN := DoesFileExist('\video\' + Filename + '.roq');

If FN <> '' then
begin
CinimaticID := LoadRoq(FN,RenderToScreen); // If RenderToScreen is false, we have to manualy render it

Cinematics.Cinematic[CinimaticID].State := State; // Make it play
Cinematics.Cinematic[CinimaticID].KeepOpen := KeepOpen; // We want it to close at the end
Cinematics.Cinematic[CinimaticID].Loop := Loop; // We want it to loop over and over!
Result := CinimaticID;
end
else
{$IfDef HELIOS_CONSOLE}cons.AddMsg('Roq File Not Found - ' + Filename);{$EndIf}

end;

end.
