unit UnitMD5;
{
Unit: UnitMD5
By: Stucuk
Description: This Unit Loads MD5 Files (Doom 3 Model Format)

Special Thanks to Vipr for all his help in making me understand the md5 format
and Sascha Willems for the quaternion to matrix code
}


interface


Uses Windows,SysUtils,Geometry,Math3d,Classes,OpenGl15, {$IfDef HELIOS_APP}Helios_Global,Helios_Types{$Else}Helios_Basic,Helios_Basic_Types{$EndIf};

Type
TQ = record
X,Y,Z,W : Single;
end;

// Animation Declerations
{TMD5AnimHeader = Record
NumFrames,NumJoints,Version,
FrameRate,NumAnimatedComponents : Integer;
end;

TMD5AnimJoint = Record
Name : String;
Qx,Qy,Zz,Tx,Ty,Tz : Boolean;
end;

TMD5AnimFrame = Record
Q,T : TVector3f;
end;

TMD5Anim = Record
Header : TMD5AnimHeader;
Joints : TMD5AnimJoint;
Bounds,BaseFrame : MD5AnimFrame;
end;     }

// Mesh Declerations

TMD5Joint = record
Name : String;
Parent : integer;
Position,Quaternion : TVector3f;
Matrix : TMatrix;
end;

TMD5Vert = record
s,t : Single;
WeightId,WeightCount : Integer;
Position : TVector3f; // Generated At Runtime
Colour : TVector3f;
Normal,sTangent,tTangent : TVector3f; // Generated At Runtime
end;

TMD5Tri = record
V1,V2,V3 : Integer;
Normal : TVector3f;
CenterOfTriangle : TVector3f;
end;

TMD5Weight = record
BoneId : Integer;
Factor : Single;
Position : TVector3f;
end;

TMD5MeshHeader = record
NumVerts,NumTris,NumWeights : integer;
end;

TMD5Mesh = record
Shader : String;
ShaderID : Integer; // Generated At Runtime
Header : TMD5MeshHeader;
Verts : Array Of TMD5Vert;
Tris : Array Of TMD5Tri;
Weights : Array Of TMD5Weight;
Visible : Boolean; // Should we draw it?
end;

TMD5Header = record
MD5Version,NumJoints,NumMeshes : integer;
end;

TMD5File = record
Header : TMD5Header;
Joints : Array Of TMD5Joint;
Meshs : Array Of TMD5Mesh;
Position,CenterOfModel : TVector3f;
end;

TMD5FaceSort = Record
Distance : Single;
TriID,MeshID : Integer;
end;

Function LoadMD5(Var MD5 : TMD5File; Const Filename : String) : Boolean;
Procedure BuildMeshs(Var MD5 : TMD5File);
Procedure RenderMD5(Var MD5 : TMD5File);
Procedure RenderMD5Mesh(Var Mesh : TMD5Mesh; Const CenterOfModel,Position : TVector3f);
Procedure RenderMD5MeshTri(Var Mesh : TMD5Mesh; Const CenterOfModel,Position : TVector3f; TriId : Integer);
Procedure RenderBones(MD5 : TMD5File);

procedure BuildMD5FaceList(MD5 : TMD5File; sort : boolean);

Var
RenderMD5Bones,MD5Normals,MD5ShowTangents,MD5Textures : Boolean;
BoneColour : TVector3f;
BonePointSize : Single;

MD5Faces,MD5FacesTrans : array of TMD5FaceSort;
numMD5Faces,numMD5FacesTrans : integer;

implementation

Uses
qsortunit, Helios_Shader_System;

function Q2M4(a:Tq):TMatrix;   // quaternion to matrix 4x4
begin
with a do
 begin 
 Result[0,0] := 1 - 2*y*y - 2*z*z;
 Result[1,0] := 2*x*y - 2*w*z; 
 Result[2,0] := 2*x*z + 2*w*y; 
 Result[3,0] := 0; 

 Result[0,1] := 2*x*y + 2*w*z; 
 Result[1,1] := 1 - 2*x*x - 2*z*z; 
 Result[2,1] := 2*y*z - 2*w*x; 
 Result[3,1] := 0; 

 Result[0,2] := 2*x*z - 2*w*y; 
 Result[1,2] := 2*y*z + 2*w*x; 
 Result[2,2] := 1 - 2*x*x - 2*y*y; 
 Result[3,2] := 0; 

 Result[0,3] := 0; 
 Result[1,3] := 0; 
 Result[2,3] := 0; 
 Result[3,3] := 1; 
 end;
end;

Function GetValue(Line : Integer; StringList : TStringList) : String;
Var
First,Rest : String;
begin

GetFirstBit(FindStart(StringList.Strings[Line]),First,Rest);

GetFirstBit(FindStart(Rest),First,Rest);

Result := First;
end;

Function GetFirstValue(Line : Integer; StringList : TStringList) : String;
Var
First,Rest : String;
begin

GetFirstBit(FindStart(StringList.Strings[Line]),First,Rest);

Result := First;
end;

Function ReturnValue(Find : String; LineStart : Integer; StringList : TStringList; Var Line : Integer) : Boolean;
var
S : String;
Begin

Line := LineStart-1;
Result := False;

repeat
Inc(Line);

If Line >= StringList.Count then
exit;

S := GetFirstValue(Line,StringList);

until Ansilowercase(S) = ansilowercase(Find);

Result := True;

end;

Function ReturnValue2(Find : String; LineStart : Integer; StringList : TStringList; Var Line : Integer) : Boolean;
var
S : String;
FF,FB : Boolean;
Begin

Line := LineStart-1;
Result := False;
FF := False;
FB := False;

repeat
Inc(Line);

If Line >= StringList.Count then
exit;

S := GetFirstValue(Line,StringList);

If Ansilowercase(s) = Ansilowercase(Find) then
FF := True;

If FF then
If Pos('{',StringList.Strings[Line]) > 0 then
FB := True;

until (FF and FB);

Result := True;

end;

Function LoadJoints(Var Line : Integer; Var MD5File : TMD5File; Const StringList : TStringList) : Boolean;
var
S,First,Rest : String;
JId : Integer;
Q : TQ;
Term : Single;
begin

Result := False;

JID := 0;

Repeat
Inc(Line);

If Line >= StringList.Count then
exit;

S := GetFirstValue(Line,StringList);

If Copy(S,1,1) = '"' then
begin
GetFirstBit(FindStart(StringReplace(StringList.Strings[Line], #9,  ' ', [rfReplaceAll])),First,Rest);
MD5File.Joints[JID].Name := StringReplace(First, '"',  '', [rfReplaceAll]);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Joints[JID].Parent := StrToInt(First);
GetFirstBit(FindStart(Rest),First,Rest);// (
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Joints[JID].Position.X := StrToFloat(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Joints[JID].Position.Y := StrToFloat(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Joints[JID].Position.Z := StrToFloat(First);

GetFirstBit(FindStart(Rest),First,Rest); // )
GetFirstBit(FindStart(Rest),First,Rest); // (

GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Joints[JID].Quaternion.X := StrToFloat(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Joints[JID].Quaternion.Y := StrToFloat(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Joints[JID].Quaternion.Z := StrToFloat(First);
Q.X := MD5File.Joints[JID].Quaternion.X;
Q.Y := MD5File.Joints[JID].Quaternion.Y;
Q.Z := MD5File.Joints[JID].Quaternion.Z;

Term := 1.0 - Q.X*Q.X - Q.Y*Q.Y - Q.Z*Q.Z;
if (term < 0) then
   Q.W := 0
else
   Q.W := {-} sqrt(term);

MD5File.Joints[JID].Matrix := Q2M4(Q);
Inc(JID);
end;

Until  S = '}';

Result := True;

end;

Function LoadMesh(Var Line,MeshNo : Integer; Var MD5File : TMD5File; Const StringList : TStringList) : Boolean;
var
S,First,Rest : String;
VID,TID,WID : Integer;
begin

Result := False;

VID := 0;
TID := 0;
WID := 0;

Repeat
Inc(Line);

If Line >= StringList.Count then
exit;

S := ansilowercase(GetFirstValue(Line,StringList));

If S = 'shader' then
begin
MD5File.Meshs[MeshNo].Shader := StringReplace(GetValue(Line,StringList), '"',  '', [rfReplaceAll]);
MD5File.Meshs[MeshNo].ShaderID := -2;
ShaderManager.AddToShaderManager(MD5File.Meshs[MeshNo].Shader);
end
else
If S = 'numverts' then
begin
MD5File.Meshs[MeshNo].Header.NumVerts := StrtoInt(GetValue(Line,StringList));
SetLength(MD5File.Meshs[MeshNo].Verts,MD5File.Meshs[MeshNo].Header.NumVerts);
VID := 0;
end
else
If S = 'vert' then
begin
GetFirstBit(FindStart(StringReplace(StringList.Strings[Line], #9,  ' ', [rfReplaceAll])),First,Rest);
GetFirstBit(FindStart(Rest),First,Rest);
VID := StrToInt(First);
GetFirstBit(FindStart(Rest),First,Rest); // (
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Verts[VID].s := StrToFloat(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Verts[VID].t := StrToFloat(First);

GetFirstBit(FindStart(Rest),First,Rest); // )
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Verts[VID].WeightId := StrToInt(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Verts[VID].WeightCount := StrToInt(First);
Inc(VID);
end
else
If S = 'numtris' then
begin
MD5File.Meshs[MeshNo].Header.NumTris := StrtoInt(GetValue(Line,StringList));
SetLength(MD5File.Meshs[MeshNo].Tris,MD5File.Meshs[MeshNo].Header.NumTris);
TID := 0;
end
else
If S = 'tri' then
begin
GetFirstBit(FindStart(StringReplace(StringList.Strings[Line], #9,  ' ', [rfReplaceAll])),First,Rest);
GetFirstBit(FindStart(Rest),First,Rest);
TID := StrToInt(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Tris[TID].V1 := StrToInt(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Tris[TID].V2 := StrToInt(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Tris[TID].V3 := StrToInt(First);
Inc(TID);
end
else
If S = 'numweights' then
begin
MD5File.Meshs[MeshNo].Header.NumWeights := StrtoInt(GetValue(Line,StringList));
SetLength(MD5File.Meshs[MeshNo].Weights,MD5File.Meshs[MeshNo].Header.NumWeights);
WID := 0;
end
else
If S = 'weight' then
begin
GetFirstBit(FindStart(StringReplace(StringList.Strings[Line], #9,  ' ', [rfReplaceAll])),First,Rest);
GetFirstBit(FindStart(Rest),First,Rest);
WID := StrToInt(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Weights[WID].BoneId := StrToInt(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Weights[WID].Factor := StrToFloat(First);
GetFirstBit(FindStart(Rest),First,Rest); // (
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Weights[WID].Position.X := StrToFloat(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Weights[WID].Position.Y := StrToFloat(First);
GetFirstBit(FindStart(Rest),First,Rest);
MD5File.Meshs[MeshNo].Weights[WID].Position.Z := StrToFloat(First);
Inc(WID);
end;

Until  S = '}';
Inc(MeshNo);

Result := True;

end;

Function LoadMD5(Var MD5 : TMD5File; Const Filename : String) : Boolean;
var
S : TStringList;
Line : Integer;
MeshNo,x,NumTri : Integer;
begin
Result := False;
If Not Fileexists(Filename) then Exit;

S := TStringList.Create;
S.LoadFromFile(Filename);

Line := 0;

If not ReturnValue('md5version',Line,S,Line) then
begin
Messagebox(0,'Error: MD5Version Not Found','MD5 Load Error',0);
Exit;
end;

MD5.Header.MD5Version := StrToInt(GetValue(Line,S));

If MD5.Header.MD5Version <> 10 then
begin
Messagebox(0,'Error: MD5Version <> 10','MD5 Load Error',0);
Exit;
end;

If not ReturnValue('numjoints',Line,S,Line) then
begin
Messagebox(0,'Error: NumJoints Not Found','MD5 Load Error',0);
Exit;
end;

MD5.Header.NumJoints := StrToInt(GetValue(Line,S));
SetLength(MD5.Joints,MD5.Header.NumJoints);

If not ReturnValue('nummeshes',Line,S,Line) then
begin
Messagebox(0,'Error: NumMeshes Not Found','MD5 Load Error',0);
Exit;
end;

MD5.Header.NumMeshes := StrToInt(GetValue(Line,S));
SetLength(MD5.Meshs,MD5.Header.NumMeshes);

If not ReturnValue('joints',Line,S,Line) then
begin
Messagebox(0,'Error: Joints Not Found','MD5 Load Error',0);
Exit;
end;

If not LoadJoints(Line,MD5,S) then
begin
Messagebox(0,'Error: Loading Joints','MD5 Load Error',0);
Exit;
end;

MeshNo := 0;
NumTri := 0;

For x := 0 to MD5.Header.NumMeshes-1 do
begin
If not LoadMesh(Line,MeshNo,MD5,S) then
begin
Messagebox(0,'Error: Loading Joints','MD5 Load Error',0);
Exit;
end;
MD5.Meshs[x].Visible := True;
NumTri := NumTri + MD5.Meshs[x].Header.NumTris;
end;

BuildMeshs(MD5);
SetLength(MD5Faces,NumTri);
SetLength(MD5FacesTrans,NumTri);

S.Free;
Result := True;
end;

Function GetPosWithMatrix(Pos : TVector3f; Const Matrix : TMatrix) : TVector3f;
begin
  Result.X := ( Pos.x * Matrix[0,0] + Pos.y * Matrix[0,1] + Pos.z * Matrix[0,2] + Matrix[0,3]);
  Result.Y := ( Pos.x * Matrix[1,0] + Pos.y * Matrix[1,1] + Pos.z * Matrix[1,2] + Matrix[1,3]);
  Result.Z := ( Pos.x * Matrix[2,0] + Pos.y * Matrix[2,1] + Pos.z * Matrix[2,2] + Matrix[2,3]);
end;

Procedure AddToMaxMin(var Max,Min : TVector3f; Const Vertex : TVector3f);
begin

If Vertex.X < Min.X then
Min.X := Vertex.X;

If Vertex.Y < Min.Y then
Min.Y := Vertex.Y;

If Vertex.Z < Min.Z then
Min.Z := Vertex.Z;

If Vertex.X > Max.X then
Max.X := Vertex.X;

If Vertex.Y > Max.Y then
Max.Y := Vertex.Y;

If Vertex.Z > Max.Z then
Max.Z := Vertex.Z;

end;

Procedure BuildMD5Mesh(MD5 : TMD5File; var Mesh : TMD5Mesh; var Max,Min : TVector3f);
var
V,W,T : Integer;
Pos,Joint,MX,MN : TVector3f;
TM : TMatrix;
Weighting : Single;
begin

// Build The Vertex's
For V := 0 to Mesh.Header.NumVerts-1 do
begin
Mesh.Verts[v].Position := SetVector(0,0,0);
Mesh.Verts[v].Colour := TColorToTVector3f(RGB(Random(256),Random(256),Random(256)));

For W := 0 to Mesh.Verts[v].WeightCount -1 do
begin
Joint := MD5.Joints[Mesh.Weights[Mesh.Verts[v].WeightId + W].BoneId].Position;
TM := MD5.Joints[Mesh.Weights[Mesh.Verts[v].WeightId + W].BoneId].Matrix;
Weighting := Mesh.Weights[Mesh.Verts[v].WeightId + W].Factor;

Pos := Mesh.Weights[Mesh.Verts[v].WeightId + W].Position;
Pos := GetPosWithMatrix(Pos,TM);
Pos := AddVector(Pos,Joint);
Pos := ScaleVector(Pos,Weighting);

Mesh.Verts[v].Position := AddVector(Mesh.Verts[v].Position,Pos);

end;

For T := 0 to Mesh.Header.NumTris-1 do // Work Out Center
begin

MX := SetVector(-9999,-9999,-9999);
MN := SetVector(9999,9999,9999);

AddToMaxMin(MX,MN,Mesh.Verts[Mesh.Tris[T].V1].Position);
AddToMaxMin(MX,MN,Mesh.Verts[Mesh.Tris[T].V2].Position);
AddToMaxMin(MX,MN,Mesh.Verts[Mesh.Tris[T].V3].Position);

Mesh.Tris[T].CenterOfTriangle := GetCenterOfBounds(MX,MN);
end;

// Calculated later!
Mesh.Verts[v].sTangent := SetVector(0,0,0);
Mesh.Verts[v].tTangent := SetVector(0,0,0);

With Mesh.Verts[v] do
begin

If Position.X < Min.X then
Min.X := Position.X;

If Position.Y < Min.Y then
Min.Y := Position.Y;

If Position.Z < Min.Z then
Min.Z := Position.Z;

If Position.X > Max.X then
Max.X := Position.X;

If Position.Y > Max.Y then
Max.Y := Position.Y;

If Position.Z > Max.Z then
Max.Z := Position.Z;

end;
end;

end;

Procedure BuildMD5MeshNormals(var Mesh : TMD5Mesh; const Max,Min : TVector3f);
var
t,v : integer;
Begin

For t := 0 to Mesh.Header.NumTris-1 do
begin
Mesh.Tris[t].Normal := InvertVector(Normalize2(CrossProduct(SubtractVector(Mesh.Verts[Mesh.Tris[t].V2].Position,Mesh.Verts[Mesh.Tris[t].V1].Position),SubtractVector(Mesh.Verts[Mesh.Tris[t].V3].Position,Mesh.Verts[Mesh.Tris[t].V2].Position))));
end;

For v := 0 to Mesh.Header.NumVerts-1 do
begin
Mesh.Verts[v].Normal := SetVector(0,0,0);
For t := 0 to Mesh.Header.NumTris-1 do
If (Mesh.Tris[t].V1 = v) or (Mesh.Tris[t].V2 = v) or (Mesh.Tris[t].V3 = v) then
Mesh.Verts[v].Normal := AddVector(Mesh.Verts[v].Normal,Mesh.Tris[t].Normal);
Normalize(Mesh.Verts[v].Normal);
end;

end;

Procedure BuildMD5Tangents(var Mesh : TMD5Mesh);
var
x,v1,v2,v3  : Integer;
x1,x2,y1,y2,z1,z2,
s1,s2,t1,t2,
r : Single;
sdir,tdir : TVector3f;
begin

for x := 0 to Mesh.Header.NumTris-1 do
begin
v1 := Mesh.Tris[x].V1;
v2 := Mesh.Tris[x].V2;
v3 := Mesh.Tris[x].V3;

x1 := Mesh.Verts[v2].Position.X - Mesh.Verts[v1].Position.X;
x2 := Mesh.Verts[v3].Position.X - Mesh.Verts[v1].Position.X;
y1 := Mesh.Verts[v2].Position.Y - Mesh.Verts[v1].Position.Y;
y2 := Mesh.Verts[v3].Position.Y - Mesh.Verts[v1].Position.Y;
z1 := Mesh.Verts[v2].Position.Z - Mesh.Verts[v1].Position.Z;
z2 := Mesh.Verts[v3].Position.Z - Mesh.Verts[v1].Position.Z;

s1 := Mesh.Verts[v2].s - Mesh.Verts[v1].s;
s2 := Mesh.Verts[v3].s - Mesh.Verts[v1].s;
t1 := Mesh.Verts[v2].t - Mesh.Verts[v1].t;
t2 := Mesh.Verts[v3].t - Mesh.Verts[v1].t;

r := 1 / (s1 * t2 - s2 * t1);

sdir := Normalize2(SetVector((s1 * x2 - s2 * x1) * r, (s1 * y2 - s2 * y1) * r,(s1 * z2 - s2 * z1) * r));
tdir := Normalize2(SetVector((t2 * x1 - t1 * x2) * r, (t2 * y1 - t1 * y2) * r,(t2 * z1 - t1 * z2) * r));

Mesh.Verts[v1].sTangent := Normalize2(AddVector(Mesh.Verts[v1].sTangent,sdir));
Mesh.Verts[v1].tTangent := Normalize2(AddVector(Mesh.Verts[v1].tTangent,tdir));
Mesh.Verts[v2].sTangent := Normalize2(AddVector(Mesh.Verts[v2].sTangent,sdir));
Mesh.Verts[v2].tTangent := Normalize2(AddVector(Mesh.Verts[v2].tTangent,tdir));
Mesh.Verts[v3].sTangent := Normalize2(AddVector(Mesh.Verts[v3].sTangent,sdir));
Mesh.Verts[v3].tTangent := Normalize2(AddVector(Mesh.Verts[v3].tTangent,tdir));
end;
        
end;

Procedure BuildMeshs(Var MD5 : TMD5File);
var
X : Integer;
Max,Min : TVector3f;
begin

Max := SetVector(-9999,-9999,-9999);
Min := SetVector(9999,9999,9999);

for x := 0 to MD5.Header.NumMeshes-1 do
BuildMD5Mesh(MD5,MD5.Meshs[x],Max,Min);

MD5.CenterOfModel := GetCenterOfBounds(Max,Min);

for x := 0 to MD5.Header.NumMeshes-1 do
BuildMD5MeshNormals(MD5.Meshs[x],Max,Min);
for x := 0 to MD5.Header.NumMeshes-1 do
BuildMD5Tangents(MD5.Meshs[x]);
//end;

end;

Procedure RenderMD5MeshTri(Var Mesh : TMD5Mesh; Const CenterOfModel,Position : TVector3f; TriId : Integer);
var
v : integer;
Vertex : Array [0..2] of TBSPVertex;
sTan,tTan : Array [0..2] of TVector3f;
TempV,Colour : TVector3f;
begin

If Mesh.ShaderID = -2 then // Assign the shader
Mesh.ShaderID := ShaderManager.GetShaderId(Mesh.Shader);


If Mesh.ShaderID < 0  then
glBindTexture(GL_TEXTURE_2D,0);

sTan[0] := Mesh.Verts[Mesh.Tris[TriId].V1].sTangent;
tTan[0] := Mesh.Verts[Mesh.Tris[TriId].V1].tTangent;
sTan[1] := Mesh.Verts[Mesh.Tris[TriId].V2].sTangent;
tTan[1] := Mesh.Verts[Mesh.Tris[TriId].V2].tTangent;
sTan[2] := Mesh.Verts[Mesh.Tris[TriId].V3].sTangent;
tTan[2] := Mesh.Verts[Mesh.Tris[TriId].V3].tTangent;

With Mesh.Verts[Mesh.Tris[TriId].V1] do
begin
Vertex[0].Position.X := Position.X;
Vertex[0].Position.Z := -Position.Y;
Vertex[0].Position.Y := Position.Z;
Vertex[0].TextureCoord.X := s;
Vertex[0].TextureCoord.Y := 1-t;
Vertex[0].Color[0] := 1;
Vertex[0].Color[1] := 1;
Vertex[0].Color[2] := 1;
Vertex[0].Color[3] := 1;
Vertex[0].Normal.X := Normal.X;
Vertex[0].Normal.Z := -Normal.Y;
Vertex[0].Normal.Y := Normal.Z;
sTan[0].X := sTangent.X;
sTan[0].Y := sTangent.Z;
sTan[0].Z := -sTangent.Y;
tTan[0].X := tTangent.X;
tTan[0].Y := tTangent.Z;
tTan[0].Z := -tTangent.Y;
end;

With Mesh.Verts[Mesh.Tris[TriId].V2] do
begin
Vertex[1].Position.X := Position.X;
Vertex[1].Position.Z := -Position.Y;
Vertex[1].Position.Y := Position.Z;
Vertex[1].TextureCoord.X := s;
Vertex[1].TextureCoord.Y := 1-t;
Vertex[1].Color[0] := 1;
Vertex[1].Color[1] := 1;
Vertex[1].Color[2] := 1;
Vertex[1].Color[3] := 1;
Vertex[1].Normal.X := Normal.X;
Vertex[1].Normal.Z := -Normal.Y;
Vertex[1].Normal.Y := Normal.Z;
sTan[1].X := sTangent.X;
sTan[1].Y := sTangent.Z;
sTan[1].Z := -sTangent.Y;
tTan[1].X := tTangent.X;
tTan[1].Y := tTangent.Z;
tTan[1].Z := -tTangent.Y;
end;

With Mesh.Verts[Mesh.Tris[TriId].V3] do
begin
Vertex[2].Position.X := Position.X;
Vertex[2].Position.Z := -Position.Y;
Vertex[2].Position.Y := Position.Z;
Vertex[2].TextureCoord.X := s;
Vertex[2].TextureCoord.Y := 1-t;
Vertex[2].Color[0] := 1;
Vertex[2].Color[1] := 1;
Vertex[2].Color[2] := 1;
Vertex[2].Color[3] := 1;
Vertex[2].Normal.X := Normal.X;
Vertex[2].Normal.Z := -Normal.Y;
Vertex[2].Normal.Y := Normal.Z;
sTan[2].X := sTangent.X;
sTan[2].Y := sTangent.Z;
sTan[2].Z := -sTangent.Y;
tTan[2].X := tTangent.X;
tTan[2].Y := tTangent.Z;
tTan[2].Z := -tTangent.Y;
end;


If (Mesh.ShaderID < 0) or (Not MD5Textures) or (UseBumpMaps and MD5Normals) then
begin

glEnable(GL_COLOR);
glbegin(GL_TRIANGLES);
Colour := DivVector2(AddVector(AddVector(Mesh.Verts[Mesh.Tris[TriId].V1].Colour, Mesh.Verts[Mesh.Tris[TriId].V2].Colour),Mesh.Verts[Mesh.Tris[TriId].V3].Colour),3);

glColor3f(Colour.X,Colour.Y,Colour.Z);

If MD5Normals then
With Vertex[0] do
glNormal3f(Normal.X,Normal.Y,Normal.Z);

With Vertex[0] do
glVertex3f(Position.X,Position.Y,Position.Z);

If MD5Normals then
With Vertex[1] do
glNormal3f(Normal.X,Normal.Y,Normal.Z);

With Vertex[1] do
glVertex3f(Position.X,Position.Y,Position.Z);

If MD5Normals then
With Vertex[2] do
glNormal3f(Normal.X,Normal.Y,Normal.Z);

With Vertex[2] do
glVertex3f(Position.X,Position.Y,Position.Z);

glend;

end
else
begin

If UseBumpMaps then
begin
Vertex[0].Normal := InvertVector(Vertex[0].Normal);
Vertex[1].Normal := InvertVector(Vertex[1].Normal);
Vertex[2].Normal := InvertVector(Vertex[2].Normal);
end;

ShaderManager.DrawFaces(Vertex,CenterOfModel,Position,3,Mesh.ShaderID,MD5Normals,sTan,tTan);

glDisable(GL_DEPTH_TEST);
glDisable(GL_TEXTURE_2D);
glColor3f(BoneColour.X,BoneColour.Y,BoneColour.Z);

glPointSize(BonePointSize);
 glBegin(GL_POINTS);
  glVertex3f(LightPos.X,LightPos.Z,-LightPos.Y);
 glEnd;
glEnable(GL_TEXTURE_2D);
glEnable(GL_DEPTH_TEST);


end;


If MD5ShowTangents then
   begin
   If MD5Normals then
   begin
   glDisable(GL_LIGHT0);
   glDisable(GL_LIGHTING);
   end;
   glDisable(GL_TEXTURE_2D);
   glBegin(GL_LINES);
     for v := 0 to 2 do
     begin
     glColor3f(1,0,0);
     glVertex3fv(@Vertex[v].Position);
     TempV := AddVector(Vertex[v].Position, sTan[v]);
     glVertex3fv(@TempV);
     glColor3f(1,1,1);
     end;
     for v := 0 to 2 do
     begin
     glColor3f(0,1,0);
     glVertex3fv(@Vertex[v].Position);
     TempV := AddVector(Vertex[v].Position, tTan[v]);
     glVertex3fv(@TempV);
     glColor3f(1,1,1);
     end;
   glEnd;
   If MD5Normals then
   begin
   glEnable(GL_LIGHT0);
   glEnable(GL_LIGHTING);
   end;
   glEnable(GL_TEXTURE_2D);
   end;


end;

Procedure RenderMD5Mesh(Var Mesh : TMD5Mesh; Const CenterOfModel,Position : TVector3f);
var
X,v : integer;
Vertex : Array [0..2] of TBSPVertex;
sTan,tTan : Array [0..2] of TVector3f;
TempV,Colour : TVector3f;
begin

If Mesh.ShaderID = -2 then // Assign the shader
Mesh.ShaderID := ShaderManager.GetShaderId(Mesh.Shader);


If Mesh.ShaderID < 0  then
glBindTexture(GL_TEXTURE_2D,0);

for x := 0 to Mesh.Header.NumTris-1 do
begin
sTan[0] := Mesh.Verts[Mesh.Tris[x].V1].sTangent;
tTan[0] := Mesh.Verts[Mesh.Tris[x].V1].tTangent;
sTan[1] := Mesh.Verts[Mesh.Tris[x].V2].sTangent;
tTan[1] := Mesh.Verts[Mesh.Tris[x].V2].tTangent;
sTan[2] := Mesh.Verts[Mesh.Tris[x].V3].sTangent;
tTan[2] := Mesh.Verts[Mesh.Tris[x].V3].tTangent;

If Mesh.ShaderID < 0 then
begin

glEnable(GL_COLOR);
glbegin(GL_TRIANGLES);

Colour := DivVector2(AddVector(AddVector(Mesh.Verts[Mesh.Tris[x].V1].Colour, Mesh.Verts[Mesh.Tris[x].V2].Colour),Mesh.Verts[Mesh.Tris[x].V3].Colour),3);

glColor3f(Colour.X,Colour.Y,Colour.Z);

If MD5Normals then
With Mesh.Verts[Mesh.Tris[x].V1] do
glNormal3f(Normal.X,Normal.Z,-Normal.Y);

With Mesh.Verts[Mesh.Tris[x].V1] do
glVertex3f(Position.X,Position.Z,-Position.Y);

If MD5Normals then
With Mesh.Verts[Mesh.Tris[x].V2] do
glNormal3f(Normal.X,Normal.Z,-Normal.Y);

With Mesh.Verts[Mesh.Tris[x].V2] do
glVertex3f(Position.X,Position.Z,-Position.Y);

If MD5Normals then
With Mesh.Verts[Mesh.Tris[x].V3] do
glNormal3f(Normal.X,Normal.Z,-Normal.Y);

With Mesh.Verts[Mesh.Tris[x].V3] do
glVertex3f(Position.X,Position.Z,-Position.Y);

glend;


end
else
begin

With Mesh.Verts[Mesh.Tris[x].V1] do
begin
Vertex[0].Position.X := Position.X;
Vertex[0].Position.Z := -Position.Y;
Vertex[0].Position.Y := Position.Z;
Vertex[0].TextureCoord.X := s;
Vertex[0].TextureCoord.Y := 1-t;
Vertex[0].Color[0] := 1;
Vertex[0].Color[1] := 1;
Vertex[0].Color[2] := 1;
Vertex[0].Color[3] := 1;
Vertex[0].Normal.X := Normal.X;
Vertex[0].Normal.Z := -Normal.Y;
Vertex[0].Normal.Y := Normal.Z;
Vertex[0].Normal := InvertVector(Vertex[0].Normal);
sTan[0].X := sTangent.X;
sTan[0].Y := sTangent.Z;
sTan[0].Z := -sTangent.Y;
tTan[0].X := tTangent.X;
tTan[0].Y := tTangent.Z;
tTan[0].Z := -tTangent.Y;
end;

With Mesh.Verts[Mesh.Tris[x].V2] do
begin
Vertex[1].Position.X := Position.X;
Vertex[1].Position.Z := -Position.Y;
Vertex[1].Position.Y := Position.Z;
Vertex[1].TextureCoord.X := s;
Vertex[1].TextureCoord.Y := 1-t;
Vertex[1].Color[0] := 1;
Vertex[1].Color[1] := 1;
Vertex[1].Color[2] := 1;
Vertex[1].Color[3] := 1;
Vertex[1].Normal.X := Normal.X;
Vertex[1].Normal.Z := -Normal.Y;
Vertex[1].Normal.Y := Normal.Z;
Vertex[1].Normal := InvertVector(Vertex[1].Normal);
sTan[1].X := sTangent.X;
sTan[1].Y := sTangent.Z;
sTan[1].Z := -sTangent.Y;
tTan[1].X := tTangent.X;
tTan[1].Y := tTangent.Z;
tTan[1].Z := -tTangent.Y;
end;

With Mesh.Verts[Mesh.Tris[x].V3] do
begin
Vertex[2].Position.X := Position.X;
Vertex[2].Position.Z := -Position.Y;
Vertex[2].Position.Y := Position.Z;
Vertex[2].TextureCoord.X := s;
Vertex[2].TextureCoord.Y := 1-t;
Vertex[2].Color[0] := 1;
Vertex[2].Color[1] := 1;
Vertex[2].Color[2] := 1;
Vertex[2].Color[3] := 1;
Vertex[2].Normal.X := Normal.X;
Vertex[2].Normal.Z := -Normal.Y;
Vertex[2].Normal.Y := Normal.Z;
Vertex[2].Normal := InvertVector(Vertex[2].Normal);
sTan[2].X := sTangent.X;
sTan[2].Y := sTangent.Z;
sTan[2].Z := -sTangent.Y;
tTan[2].X := tTangent.X;
tTan[2].Y := tTangent.Z;
tTan[2].Z := -tTangent.Y;
end;

ShaderManager.DrawFaces(Vertex,CenterOfModel,Position,3,Mesh.ShaderID,MD5Normals,sTan,tTan);

If MD5ShowTangents then
   begin
   glDisable(GL_TEXTURE_2D);
   glBegin(GL_LINES);
     for v := 0 to 2 do
     begin
     glColor3f(1,0,0);
     glVertex3fv(@Vertex[v].Position);
     TempV := AddVector(Vertex[v].Position, sTan[v]);
     glVertex3fv(@TempV);
     glColor3f(1,1,1);
     end;
     for v := 0 to 2 do
     begin
     glColor3f(0,1,0);
     glVertex3fv(@Vertex[v].Position);
     TempV := AddVector(Vertex[v].Position, tTan[v]);
     glVertex3fv(@TempV);
     glColor3f(1,1,1);
     end;
   glEnd;
   glEnable(GL_TEXTURE_2D);
   end;

glDisable(GL_DEPTH_TEST);
glDisable(GL_TEXTURE_2D);
glColor3f(BoneColour.X,BoneColour.Y,BoneColour.Z);

glPointSize(BonePointSize);
 glBegin(GL_POINTS);
  glVertex3f(LightPos.X,LightPos.Z,-LightPos.Y);
 glEnd;
glEnable(GL_TEXTURE_2D);
glEnable(GL_DEPTH_TEST);


end;

end;

end;

Procedure RenderMD5(Var MD5 : TMD5File);
var
x : integer;
begin

If MD5Normals then
begin
glEnable(GL_LIGHT0);
glEnable(GL_LIGHTING);
end;

// builds up MD5Faces and MD5FacesTrans so they are ordered by depth
BuildMD5FaceList(MD5,True);

// render "solid" faces
If numMD5Faces > 0 then
For x := 0 to numMD5Faces-1 do
If MD5.Meshs[MD5Faces[x].MeshID].Visible then
RenderMD5MeshTri(MD5.Meshs[MD5Faces[x].MeshID],MD5.CenterOfModel,MD5.Position,MD5Faces[x].TriID);

// render faces that have a form of transparency
If numMD5FacesTrans > 0 then
For x := 0 to numMD5FacesTrans-1 do
If MD5.Meshs[MD5FacesTrans[x].MeshID].Visible then
RenderMD5MeshTri(MD5.Meshs[MD5FacesTrans[x].MeshID],MD5.CenterOfModel,MD5.Position,MD5FacesTrans[x].TriID);
             

If MD5Normals then
begin
glDisable(GL_LIGHT0);
glDisable(GL_LIGHTING);
end;

If RenderMD5Bones then
RenderBones(MD5);

end;

Procedure RenderBones(MD5 : TMD5File);
var
x : integer;
begin
glDisable(GL_DEPTH_TEST);
glDisable(GL_TEXTURE_2D);
glColor3f(BoneColour.X,BoneColour.Y,BoneColour.Z);

glPointSize(BonePointSize);
for x := 0 to MD5.Header.NumJoints-1 do
begin
 glBegin(GL_POINTS);
  with MD5.Joints[x] do
  glVertex3f(Position.X,Position.Z,-Position.Y);
 glEnd;
end;
glEnable(GL_TEXTURE_2D);
glEnable(GL_DEPTH_TEST);
end;

function CompareMD5Faces(e1, e2: word): integer; far;
begin
  if MD5Faces[e1].distance < MD5Faces[e2].distance then
    result := -1
  else if MD5Faces[e1].distance > MD5Faces[e2].distance then
    result := 1
  else
    result := 0;
end;

procedure SwapMD5Faces(e1, e2: word); far;
var s: TMD5FaceSort;
begin
  s := MD5Faces[e1];
  MD5Faces[e1] := MD5Faces[e2];
  MD5Faces[e2] := s;
end;

function CompareMD5FacesTrans(e1, e2: word): integer; far;
begin
  if MD5FacesTrans[e1].distance < MD5FacesTrans[e2].distance then
    result := -1
  else if MD5FacesTrans[e1].distance > MD5FacesTrans[e2].distance then
    result := 1
  else
    result := 0;
end;

procedure SwapMD5FacesTrans(e1, e2: word); far;
var s: TMD5FaceSort;
begin
  s := MD5FacesTrans[e1];
  MD5FacesTrans[e1] := MD5FacesTrans[e2];
  MD5FacesTrans[e2] := s;
end;

// Works out the order the Faces should be drawn
procedure BuildMD5FaceList(MD5 : TMD5File; sort : boolean);
var TriIndex,MeshIndex, shaderId, x : integer;
    distance : single;
    IsTransparent : boolean;
begin
numMD5Faces := 0;
numMD5FacesTrans := 0;

  for MeshIndex := 0 to MD5.Header.NumMeshes-1 do
  for TriIndex := 0 to MD5.Meshs[MeshIndex].Header.NumTris-1 do begin

    if sort then begin
      distance := VectorDistance(MD5.Meshs[MeshIndex].Tris[TriIndex].CenterOfTriangle,{$IfDef HELIOS_APP}Camera.GetCurrentPosition{$Else}GetCurrentPosition{$EndIf});
      distance := - distance;
      If Distance > 0 then
      distance := - distance;
    end
    else
      distance := 0;

      If MD5.Meshs[MeshIndex].ShaderID = -2 then // Assign the shader
      MD5.Meshs[MeshIndex].ShaderID := ShaderManager.GetShaderId(MD5.Meshs[MeshIndex].Shader);

      shaderId := MD5.Meshs[MeshIndex].ShaderID;

      if  shaderId > -1 then begin

        if (ShaderManager[shaderId].Surface and 2048) =  2048 then continue; // nodraw

        if ShaderManager[shaderId].IsSkyBox then
          continue;

        IsTransparent := False;
        If shaderManager[shaderId].numOfLayers > 0 then
        For x := 0 to shaderManager[shaderId].numOfLayers-1 do
        If shaderManager[shaderId].Layers[x].Draw then
        If shaderManager[shaderId].Layers[x].doAlphaTest or shaderManager[shaderId].Layers[x].doBlending then
        IsTransparent := True;

        if (shaderManager[shaderId].hasFog) or (ShaderManager[shaderId].TotallyTrans) or (ShaderManager[shaderId].Cull = GL_NONE) or IsTransparent then
        Begin
          MD5FacesTrans[numMD5FacesTrans].TriID := TriIndex;
          MD5FacesTrans[numMD5FacesTrans].MeshID := MeshIndex;
          MD5FacesTrans[numMD5FacesTrans].distance := distance;

          if (ShaderManager[shaderId].PolygonOffset) then
          if not (shaderManager[shaderId].hasFog) then
          MD5FacesTrans[numMD5FacesTrans].distance := MD5FacesTrans[numMD5FacesTrans].distance + 0.01;
          // End of Fix
          inc(numMD5FacesTrans);
        end
        else
        begin
          MD5Faces[numMD5Faces].TriID := TriIndex;
          MD5Faces[numMD5Faces].MeshID := MeshIndex;
          MD5Faces[numMD5Faces].distance := distance;

          if (ShaderManager[shaderId].PolygonOffset) then
          if not (shaderManager[shaderId].hasFog) then
          MD5Faces[numMD5Faces].distance := MD5Faces[numMD5Faces].distance + 0.01;
          // End of Fix
          inc(numMD5Faces);
        end;
        end
        else
        begin
          MD5Faces[numMD5Faces].TriID := TriIndex;
          MD5Faces[numMD5Faces].MeshID := MeshIndex;
          MD5Faces[numMD5Faces].distance := distance;

          inc(numMD5Faces);
        end;
  end;

  If Sort then
  begin
  QSort(numMD5Faces, CompareMD5Faces, SwapMD5Faces);
  QSort(numMD5FacesTrans, CompareMD5FacesTrans, SwapMD5FacesTrans);
  end;
end;

Begin
BoneColour := SetVector(1,1,0);
BonePointSize := 5;
MD5Normals := False;
MD5Textures := True;
end.
