unit math3d;
{
Cut down version of Math3d.Pas
}

interface

type TVector3f = record
  X, Y, Z : single;
end;

type TVector3i = record
  X, Y, Z : integer;
end;

type TVector3b = record
  R, G, B : byte;
end;

type TVector3b2 = record
  X, Y, Z : byte;
end;

type TVector3si = record
  X, Y, Z : smallint;
end;

type TVector3l = record
  X, Y, Z : longword;
end;

type TVector2f = record
  X, Y : single;
end;

type TVector2i = record
  X, Y : Integer;
end;

Type TVector4f = record
  X, Y, Z, Alpha : single;
end;

  PSmallIntArray = ^TSmallIntArray;
  TSmallIntArray = array[0..MAXINT shr 2] of SmallInt;

  PCardinalArray = ^TCardinalArray;
  TCardinalArray = array[0..MAXINT shr 3] of Cardinal;

// TGLPixel32
   //
   TGLPixel32 = packed record
      r, g, b, a : Byte;
   end;

   PGLPixel32Array = ^TGLPixel32Array;
   TGLPixel32Array = array of TGLPixel32;

   // General arrays
   PByteArray = ^TByteArray;
   TByteArray = array[0..32767] of Byte;

const
  M_PI = 3.1415926535897932384626433832795;		// matches value in gcc v2 math.h
  M_TWO_PI = 6.283185307179586476925286766559;
  M_180_PI = 57.295779513082320876798154814105;
  M_PI_180 = 0.017453292519943295769236907684886;

function SubtractVector(v1, v2 : TVector3f) : TVector3f;
function AddVector(v1, v2 : TVector3f) : TVector3f;
function SetVector(x, y, z : single) : TVector3f;
function Normalize(var v : TVector3f) : single;
function ScaleVector(v : TVector3f; s : single) : TVector3f;
Function GetCenterOfBounds(Bmax,Bmin : TVector3f) : TVector3f;
Function DivVector2(V1 : TVector3f; V2 : Extended) : TVector3f;
function InvertVector(v : TVector3f) : TVector3f;
function Normalize2(v : TVector3f) : TVector3f;
function CrossProduct(const V1, V2: TVector3f): TVector3f;
function VectorDistance(const v1, v2 : TVector3f) : Single; register;

implementation

function SubtractVector(v1, v2 : TVector3f) : TVector3f;
begin
  with result do begin
    x := v1.x - v2.x;
    y := v1.y - v2.y;
    z := v1.z - v2.z;
  end;
end;

function AddVector(v1, v2 : TVector3f) : TVector3f;
begin
  with result do begin
    x := v1.x + v2.x;
    y := v1.y + v2.y;
    z := v1.z + v2.z;
  end;
end;

function SetVector(x, y, z : single) : TVector3f;
begin
  result.x := x;
  result.y := y;
  result.z := z;
end;

function Normalize(var v : TVector3f) : single;
var l : single;
begin
  l := sqrt(v.x*v.x+v.y*v.y+v.z*v.z);

  if (l > 0) then begin
    v.x := v.x/l;
    v.y := v.y/l;
    v.z := v.Z/l;
  end;
  result := l;  
end;

function ScaleVector(v : TVector3f; s : single) : TVector3f;
begin
  with result do begin
    x := v.x * s;
    y := v.y * s;
    z := v.z * s;
  end;
end;

Function DivVector2(V1 : TVector3f; V2 : Extended) : TVector3f;
begin

Result := SetVector(0,0,0);

if V2 <> 0 then
Result.X := V1.X / V2;

if V2 <> 0 then
Result.Y := V1.Y / V2;

if V2 <> 0 then
Result.Z := V1.Z / V2;

end;

Function GetCenterOfBounds(Bmax,Bmin : TVector3f) : TVector3f;
begin
Result := AddVector(DivVector2(SubtractVector(Bmax,Bmin),2),BMin);
end;

function InvertVector(v : TVector3f) : TVector3f;
begin
  with result do begin
    x := -v.x;
    y := -v.y;
    z := -v.z;
  end;
end;

function Normalize2(v : TVector3f) : TVector3f;
var l : single;
begin
  l := sqrt(v.x*v.x+v.y*v.y+v.z*v.z);

  if (l > 0) then begin
    v.x := v.x/l;
    v.y := v.y/l;
    v.z := v.Z/l;
  end;
  result := v;
end;

function CrossProduct(const V1, V2: TVector3f): TVector3f;
begin
   Result.X:=V1.Y * V2.z - V1.z * V2.y;
   Result.Y:=V1.z * V2.x - V1.x * V2.z;
//   Result.Z:=V1.x * V2.y - V1.x * V2.x;
   Result.Z:=V1.x * V2.y - V1.y * V2.x;
end;

function VectorDistance(const v1, v2 : TVector3f) : Single; register;
// EAX contains address of v1
// EDX contains highest of v2
// Result  is passed on the stack
asm
      FLD  DWORD PTR [EAX]
      FSUB DWORD PTR [EDX]
      FMUL ST, ST
      FLD  DWORD PTR [EAX+4]
      FSUB DWORD PTR [EDX+4]
      FMUL ST, ST
      FADD
      FLD  DWORD PTR [EAX+8]
      FSUB DWORD PTR [EDX+8]
      FMUL ST, ST
      FADD
      FSQRT
end;

end.
