unit shaderValues;

interface

uses OpenGL15, classes, sysutils, Hashes{$IfDef HELIOS_APP}, Helios_Types{$EndIf};

var ValueList : TIntegerHash;
function LookUp(name : string) : integer;
Function LookUpHeliosSurface(name : string) : Integer;

implementation

uses Dialogs{$IfDef HELIOS_APP}, Helios_Shaders{$Else}, Helios_Shader_System{$EndIf};

{$IfDef HELIOS_APP}
{$Else}
const FACE_POLYGON   = 1;
      FACE_PATCH     = 2;
      FACE_MESH      = 3;
      FACE_BILLBOARD = 4;

      DIST_EPSILON = 1/32; //0.000001;

      Q3_FONT = 0;
      Q3_BACK = 1;

      CONTENTS_SOLID = 1; // an eye is never valid in a solid
      CONTENTS_WINDOW = 2; // translucent, but not watery
      CONTENTS_AUX = 4;
      CONTENTS_LAVA = 8;
      CONTENTS_SLIME = 16;
      CONTENTS_WATER = 32;
      CONTENTS_FOG = 64;
      LAST_VISIBLE_CONTENTS = 64;

      // remaining contents are non-visible, and don't eat brushes
      CONTENTS_AREAPORTAL =    $8000;
      CONTENTS_PLAYERCLIP =    $10000;
      CONTENTS_MONSTERCLIP =   $20000;
      CONTENTS_TELEPORTER  =   $40000;
      CONTENTS_JUMPPAD	   =   $80000;
      CONTENTS_CLUSTERPORTAL = $100000;
      CONTENTS_DONOTENTER  =   $200000;
      CONTENTS_BOTCLIP	   =   $400000;
      CONTENTS_MOVER	   =   $800000;

      CONTENTS_ORIGIN =      $1000000; // removed before bsping an entity
      CONTENTS_MONSTER =     $2000000; // should never be on a brush, only in game
      CONTENTS_DEADMONSTER = $4000000;
      CONTENTS_DETAIL =      $8000000; // brushes to be added after vis leafs
      CONTENTS_STRUCTURAL  = $10000000;
      CONTENTS_TRANSLUCENT = $20000000; // auto set if any surface has trans
      CONTENTS_LADDER =      $20000000;
      CONTENTS_NODROP =      $80000000;
                             
      MASK_PLAYERSOLID = CONTENTS_SOLID + CONTENTS_PLAYERCLIP + CONTENTS_MONSTER + CONTENTS_WINDOW ;

      // Following flags are used in surface_flags (cf. shader)
      SURF_NODAMAGE	   = $1;	// never give falling damage
      SURF_SLICK	   = $2;	// effects game physics
      SURF_SKY		   = $4;	// lighting from environment map
      SURF_LADDER	   = $8;
      SURF_NOIMPACT	   = $10;	// don't make missile explosions
      SURF_NOMARKS	   = $20;	// don't leave missile marks
      SURF_FLESH	   = $40;	// make flesh sounds and effects
      SURF_NODRAW	   = $80;	// don't generate a drawsurface at all
      SURF_HINT		   = $100;	// make a primary bsp splitter
      SURF_SKIP		   = $200;	// completely ignore, allowing non-closed brushes
      SURF_NOLIGHTMAP	   = $400;	// surface doesn't need a lightmap
      SURF_POINTLIGHT      = $800;	// generate lighting info at vertexes
      SURF_METALSTEPS	   = $1000;	// clanking footsteps
      SURF_NOSTEPS	   = $2000;	// no footstep sounds
      SURF_NONSOLID	   = $4000;	// don't collide against curves with this set
      SURF_LIGHTFILTER	   = $8000;	// act as a light filter during q3map -light
      SURF_ALPHASHADOW	   = $10000;	// do per-pixel light shadow casting in q3map
      SURF_NODLIGHT	   = $20000;	// don't light even if solid (solid lava, skies)
      SURF_DUST		   = $40000;    // leave a dust trail when walking on this surface

// Helios Surfaces
      HSURF_LADDER         = $2;
      HSURF_WATER          = $4;
      HSURF_SLIME          = $8;
      HSURF_LAVA           = $10;
{$EndIf}

function LookUp(name : string) : integer;
begin
  result := -1;
  if (Copy(name, 1, 1) >= '0') and (Copy(name, 1, 1) <= '9') then begin
    result := StrToInt(name);
    exit;
  end;

  try
    if ValueList.Exists(name) then
      result := ValueList[name]
  except
    ShowMessage('value ' + name + ' not found in lookup list!');
    result := -1;
  end;
end;

Function LookUpHeliosSurface(name : string) : Integer;
var
N : String;
begin

N := AnsiLowerCase(Name);

Result := 0;

If N = 'ladder' then
Result := HSURF_LADDER
else
If N = 'water' then
Result := HSURF_WATER
else
If N = 'slime' then
Result := HSURF_SLIME
else
If N = 'lava' then
Result := HSURF_LAVA;

end;

initialization
  ValueList := TIntegerHash.Create;
  // CULL
  ValueList['front'] := GL_FRONT;
  ValueList['back'] := GL_BACK;
  ValueList['backsided'] := GL_BACK;
  ValueList['disable'] := GL_NONE;
  ValueList['none'] := GL_NONE;
  ValueList['twosided'] := GL_NONE;
  // SORT
  ValueList['portal'] := Ord(SK_PORTAL);
  ValueList['sky'] := Ord(SK_SKY);
  ValueList['opaque'] := Ord(SK_OPAQUE);
  ValueList['banner'] := Ord(SK_BANNER);
  ValueList['trans'] := Ord(SK_TRANS);
  ValueList['underwater'] := Ord(SK_UNDERWATER);
  ValueList['additive'] := Ord(SK_ADDITIVE);
  ValueList['nearest'] := Ord(SK_NEAREST);
  // depth func
  ValueList['equal'] := GL_EQUAL;
  ValueList['lequal'] := GL_LEQUAL;
  // alpha func
  ValueList['gt0'] := GL_GREATER;
  ValueList['lt128'] := GL_LESS;
  ValueList['ge128'] := GL_GEQUAL;
  ValueList['noalpha'] := GL_ALWAYS;
  // tcmod func
  ValueList['unused'] := Ord(TCMOD_UNUSED);
  ValueList['rotate'] := Ord(TCMOD_ROTATE);
  ValueList['scale'] := Ord(TCMOD_SCALE);
  ValueList['scroll'] := Ord(TCMOD_SCROLL);
  ValueList['stretch'] := Ord(TCMOD_STRETCH);
  ValueList['transform'] := Ord(TCMOD_TRANSFORM);
  ValueList['turb'] := Ord(TCMOD_TURB);
  // wave funcs
  ValueList['sin'] := Ord(WF_SIN);
  ValueList['triangle'] := Ord(WF_TRIANGLE);
  ValueList['square'] := Ord(WF_SQUARE);
  ValueList['sawtooth'] := Ord(WF_SAWTOOTH);
  ValueList['inversesawtooth'] := Ord(WF_INVSAWTOOTH);
  ValueList['noise'] := Ord(WF_NOISE);
  // rgbGen
  ValueList['identitylighting'] := Ord(RGBGEN_IDENTITYLIGHTNING);
  ValueList['identity'] := Ord(RGBGEN_IDENTITY);
  ValueList['wave'] := Ord(RGBGEN_WAVE);
  ValueList['entity'] := Ord(RGBGEN_ENTITY);
  ValueList['oneminusentity'] := Ord(RGBGEN_ONEMINUSENTITY);
  ValueList['vertex'] := Ord(RGBGEN_VERTEX);
  ValueList['exactvertex'] := Ord(RGBGEN_VERTEX);
  ValueList['oneminusvertex'] := Ord(RGBGEN_ONEMINUSVERTEX);
  ValueList['lightingdiffuse'] := Ord(RGBGEN_LIGHTNINGDIFFUSE);
  // tcGen
  ValueList['environment'] := Ord(TCGEN_ENVIRONMENT);
  ValueList['base'] := Ord(TCGEN_BASE);
  ValueList['lightmap'] := Ord(TCGEN_LIGHTMAP);
  ValueList['vector'] := Ord(TCGEN_VECTOR);
  // blending
  ValueList['gl_zero'] := GL_ZERO;
  ValueList['gl_one'] := GL_ONE;
  ValueList['gl_src_color'] := GL_SRC_COLOR;
  ValueList['gl_one_minus_src_color'] := GL_ONE_MINUS_SRC_COLOR;
  ValueList['gl_src_alpha'] := GL_SRC_ALPHA;
  ValueList['gl_one_minus_src_alpha'] := GL_ONE_MINUS_SRC_ALPHA;
  ValueList['gl_dst_alpha'] := GL_DST_ALPHA;
  ValueList['gl_one_minus_dst_alpha'] := GL_ONE_MINUS_DST_ALPHA;
  ValueList['gl_dst_color'] := GL_DST_COLOR;
  ValueList['gl_one_minus_dst_color'] := GL_ONE_MINUS_DST_COLOR;
  ValueList['gl_src_alpha_saturate'] := GL_SRC_ALPHA_SATURATE;
  // DEFORMVERTEXES
  ValueList['def_wave'] := Ord(DEFORMVERTEXES_WAVE);
  ValueList['def_normal'] := Ord(DEFORMVERTEXES_NORMAL);
  ValueList['def_bulge'] := Ord(DEFORMVERTEXES_BULGE);
  ValueList['def_move'] := Ord(DEFORMVERTEXES_MOVE);
  ValueList['def_autosprite'] := Ord(DEFORMVERTEXES_AUTOSPRITE);
  ValueList['def_autosprite2'] := Ord(DEFORMVERTEXES_AUTOSPRITE2);

  // SURFACEPARM
  ValueList['alphashadow']   := 2;
  ValueList['areaportal']    := 4;
  ValueList['clusterportal'] := 8;
  ValueList['donotenter']    := 16;
  ValueList['flesh']         := 32;
  ValueList['fog']           := 64;
  ValueList['lava']          := 128;
  ValueList['metalsteps']    := 256;
  ValueList['nodamage']      := 512;
  ValueList['nodlight']      := 1024;
  ValueList['nodraw']        := 2048;
  ValueList['nodrop']        := 4096;
  ValueList['noimpact']      := 8192;
  ValueList['nomarks']       := 16384;
  ValueList['nolightmap']    := 32768;
  ValueList['nosteps']       := 65536;
  ValueList['nonsolid']      := 131072;
  ValueList['origin']        := 262144;
  ValueList['playerclip']    := 524288;
  ValueList['slick']         := 1048576;
  ValueList['slime']         := 2097152;
  ValueList['structural']    := 4194304;
  ValueList['trans']         := 8388608;
  ValueList['water']         := 16777216;
  ValueList['detail']        := 33554432;
  ValueList['nomipmaps']     := 67108864;
  ValueList['sky']           := 134217728;
  ValueList['hint']          := 1; // @@@ check value

// RTCW
//	{"metal",	0,	SURF_METAL,0 },
//	{"woodsteps",	0,	SURF_WOOD,0 },
//	{"grasssteps",	0,	SURF_GRASS,0 },
//	{"gravelsteps",	0,	SURF_GRAVEL,0 },
//	{"snowsteps",	0,	SURF_SNOW,0 },
//	{"roofsteps",	0,	SURF_ROOF,0 },	// tile roof type sound


  ValueList['woodsteps']     := 0;
  ValueList['snowsteps']     := 0;
  ValueList['roofsteps']     := 0;
  ValueList['gravelsteps']   := 0;
  ValueList['grasssteps']    := 0;
  ValueList['metal']         := 256;

  ValueList['monsterclip']         := 524288;//0;
  ValueList['ladder']              := 0;
  ValueList['clipmissile']         := 0;
  ValueList['monsterslick']        := 0;
  ValueList['monsterslicknorth']   := 0;
  ValueList['monsterslickeast']    := 0;
  ValueList['monsterslicksouth']   := 0;
  ValueList['monsterslickwest']    := 0;
  ValueList['glass']               := 0;
  ValueList['carpetsteps']         := 0;
  
// JKA
  ValueList['nonopaque']           := 0;


finalization
  ValueList.Free;
end.
