unit FormMain;

{$I Global_Conditionals.INC}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, Menus, StdCtrls, ImgList, Buttons, Spin, ShellAPI,
  math3d, jpeg, Helios_ROQ_Engine, Helios_Timer, Helios_Basic,Helios_Sound;

Const
APPLICATION_TITLE = 'Project Helios: Roq Viewer';
APPLICATION_VER = '1.1';
APPLICATION_BY = 'Stucuk';

type
  TFrmMain = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Panel1: TPanel;
    StatusBar1: TStatusBar;
    MainView: TPanel;
    Help1: TMenuItem;
    About1: TMenuItem;
    Options1: TMenuItem;
    Disable3DView1: TMenuItem;
    TopBarImageHolder: TImage;
    Label3: TLabel;
    Label21: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Panel2: TPanel;
    LoadModel1: TMenuItem;
    N2: TMenuItem;
    OpenROQDialog: TOpenDialog;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    LoopBox: TCheckBox;
    Quality: TComboBox;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Idle(Sender: TObject; var Done: Boolean);
    Procedure SetCaption(Filename : String);
    Procedure SetIsEditable;
    procedure MainViewResize(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Disable3DView1Click(Sender: TObject);
    procedure LoadModel1Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure LoopBoxClick(Sender: TObject);
    procedure QualityChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Loading : Boolean;
    RoqLoaded : Boolean;
  end;

var
  FrmMain: TFrmMain;

implementation

uses FormAboutNew,ThreeDView_Engine;

{$R *.dfm}

procedure TFrmMain.Exit1Click(Sender: TObject);
begin
Close;
end;

procedure TFrmMain.Idle(Sender: TObject; var Done: Boolean);
begin
if (not MVEnabled) or (not RoqLoaded) then exit;

DrawFrames;

StatusBar1.Panels[0].Text := 'State: No Roq Open';

If (CurrRoq > -1) and (CurrRoq < Cinematics.Cinematic_No) then
if IsRoqActive(Cinematics.Cinematic[CurrRoq]) then
if Cinematics.Cinematic[CurrRoq].State = 0 then
StatusBar1.Panels[0].Text := 'State: Stopped'
else
if Cinematics.Cinematic[CurrRoq].State = 1 then
StatusBar1.Panels[0].Text := 'State: Playing'
else
if Cinematics.Cinematic[CurrRoq].State = 2 then
StatusBar1.Panels[0].Text := 'State: Paused';

Done := false;
end;

procedure TFrmMain.FormCreate(Sender: TObject);
begin
Loading := True;
SetCaption('');
MainViewWidth := MainView.Width;
MainViewHeight := MainView.Height;

Application.OnIdle := Idle;
Init(MainView.Handle);

SetIsEditable;

Quality.ItemIndex := TextureQuality; // Set To Default

Loading := False;
end;

Procedure TFrmMain.SetCaption(Filename : String);
begin

If Filename <> '' then
Caption := ' ' + APPLICATION_TITLE + ' v'+APPLICATION_VER + ' [' +Extractfilename(Filename) + ']'
else
Caption := ' ' + APPLICATION_TITLE + ' v'+APPLICATION_VER;

end;

Procedure TFrmMain.SetIsEditable;
var
Value : Boolean;
begin
Value := RoqLoaded;

Options1.Visible := Value;

SpeedButton1.Enabled := Value;
SpeedButton2.Enabled := Value;
SpeedButton3.Enabled := Value;
LoopBox.Enabled := Value;

if Value then
MainView.Cursor := crCross
else
MainView.Cursor := crDefault;

end;

procedure TFrmMain.MainViewResize(Sender: TObject);
begin
if oglloaded then
ResizeMainView(MainView.Width,MainView.Height);
end;

procedure TFrmMain.About1Click(Sender: TObject);
var
  frm: TFrmAbout_New;
begin
  frm:=TFrmAbout_New.Create(Self);
  frm.Visible:=False;
  Frm.Image1.Picture := TopBarImageHolder.Picture;
  frm.Label2.Caption := APPLICATION_TITLE + ' v' + APPLICATION_VER;
  frm.label1.Caption := frm.label1.Caption + APPLICATION_BY;
  frm.ShowModal;
  frm.Close;
  frm.Free;
end;

procedure TFrmMain.Disable3DView1Click(Sender: TObject);
begin
MVEnabled := not MVEnabled;
Disable3DView1.Checked := not MVEnabled;

if not MVEnabled then // no point refreshing if opengl is on
MainView.Refresh;
end;

procedure TFrmMain.LoadModel1Click(Sender: TObject);
var
SID : Integer;
begin

If OpenROQDialog.Execute then
begin
CurrRoq := LoadRoq(OpenROQDialog.FileName,True);
RoqLoaded := IsRoqActive;
If RoqLoaded then
begin
SetIsEditable;
SetCaption(OpenROQDialog.FileName);
gTimer.Refresh;
Cinematics.Cinematic[CurrRoq].State := 1;
Cinematics.Cinematic[CurrRoq].KeepOpen := True;
Cinematics.Cinematic[CurrRoq].Loop := LoopBox.Checked;
{$IfDEF HELIOS_SOUND}
SID := Sound.NewSample(FullScreenRoqMode);
Cinematics.Cinematic[CurrRoq].FmodEmitterID := Sound.NewEmitter(-1,SID,SetVector(0,0,0),False,False);
{$ENDIF}
end
else
SetCaption('');
end;

end;

procedure TFrmMain.SpeedButton1Click(Sender: TObject);
begin
if (CurrRoq < 0) or (CurrRoq >= Cinematics.Cinematic_No) then exit;
if not IsRoqActive(Cinematics.Cinematic[CurrRoq]) then exit;

gTimer.Refresh;
Cinematics.Cinematic[CurrRoq].State := 1;
end;

procedure TFrmMain.SpeedButton3Click(Sender: TObject);
begin
if (CurrRoq < 0) or (CurrRoq >= Cinematics.Cinematic_No) then exit;
if not IsRoqActive(Cinematics.Cinematic[CurrRoq]) then exit;
gTimer.Refresh;

If Cinematics.Cinematic[CurrRoq].State = 1 then
Cinematics.Cinematic[CurrRoq].State := 2
else
Cinematics.Cinematic[CurrRoq].State := 1;
end;

procedure TFrmMain.SpeedButton2Click(Sender: TObject);
begin
if (CurrRoq < 0) or (CurrRoq >= Cinematics.Cinematic_No) then exit;
if not IsRoqActive(Cinematics.Cinematic[CurrRoq]) then exit;

StopRoq(Cinematics.Cinematic[CurrRoq]);
end;

procedure TFrmMain.LoopBoxClick(Sender: TObject);
begin
if (CurrRoq < 0) or (CurrRoq >= Cinematics.Cinematic_No) then exit;
if not IsRoqActive(Cinematics.Cinematic[CurrRoq]) then exit;

Cinematics.Cinematic[CurrRoq].Loop := LoopBox.Checked;
end;

procedure TFrmMain.QualityChange(Sender: TObject);
begin
TextureQuality := Quality.ItemIndex;
end;

end.
