unit Helios_Basic;
{
Description: Used to emulate basic helios things.
}

interface

Uses Windows,Graphics,Math3d,OpenGl15,Helios_Timer, Helios_VertexFragmentProgram,
     qne_RoQ, Helios_Sound;

//Helios_Global.Pas
Const
HELIOS_MODFOLDER = 'Main';
DEPTH_OF_VIEW = 4000;
FOV = 45;

Var
GAME_FOLDER,MOD_FOLDER,BaseDir : String;
SCREEN_WIDTH,SCREEN_HEIGHT : Integer;
lastError : Cardinal;
lastErrorString : String;
h_DC   : HDC;
h_RC   : HGLRC;
h_Wnd  : HWND;
UseCameraByMouse : Boolean; // Not Used
EnvironmentPos : TVector3f;
FakeLightmap : TVector3f;
FakeLightmapB : Boolean;
Pause : Boolean;
LastTexID : Integer;
Sound : TSoundEngine;
FX_VOL : integer = 255;
MUSIC_VOL : integer = 255;

// Note Not all of these are needed
MUSIC_Channel, MUSIC_SND1, MUSIC_SND2, MUSIC_SND3 : integer;
MUSIC_Loop,MUSIC_Start : boolean;

FX_Player_Sound : array of integer;
FX_Player_Sound_No : integer;
FX_Player_Sound_Channel : integer;
FX_Death1,FX_Death2,FX_Death3,FX_Drown,FX_Fall1,FX_Falling1,FX_Gasp,FX_Jump,FX_Pain_25_1,
FX_Pain_50_1,FX_Pain_75_1,FX_Pain_100_1,FX_Taunt : integer;
FX_Sound_Channel : Integer;
FXS : Integer = -1;
//

TextureQuality : integer = 1; // 0 = Bilinear 1 = Trilinear
LastTexQuality : integer = 1;

//Helios_Types.Pas
type TBSPVertex = record
  Position      : TVector3f;             // (x, y, z) position.
  TextureCoord  : TVector2f;             // (u, v) texture coordinate
  LightmapCoord : TVector2f;             // (u, v) lightmap coordinate
  Normal        : TVector3f;             // (x, y, z) normal vector
  Color         : array[0..3] of Byte    // RGBA color for the vertex
end;

TRGBA = array[0..3] of GLUbyte;

//Q3BSP.Pas
var fogBuffer : array of single;
    inputBuffer : array[0..16383] of TBSPVertex;
    texCoordBuffer :  array[0..16383] of TVector2f;
    colorBuffer : array[0..16383] of TRGBA;

//Helios_Global.Pas
function StrToFloat(s : string) : extended;
Function TColorToTVector3f(Color : TColor) : TVector3f;
Function TVector3fToTColor(Vector3f : TVector3f) : TColor;

//Helios_Engine.Pas
Procedure ChangeModFolder(ModName : string);
Procedure InitiateHelios(GameFolder,ModFolder, Params : String);
procedure glResizeWnd(Width, Height : Integer);
Procedure IsGame;
Procedure DrawGfxToScreen(Texture : Cardinal; X,Y,Width,Height,Alpha : Single; Colour : TVector3f);

//Helios_FileSystem.Pas
Function DoesFileExist(Filename : String) : String;

//Q3BSP.Pas
procedure SetDefaults;

implementation

Uses SysUtils;

//Helios_Global.Pas
function StrToFloat(s : string) : extended;
begin
  try
   // s := StringReplace(s, '.', ',', [rfReplaceAll]);
    result := sysutils.StrToFloat(s);
  except
  Try
    s := StringReplace(s, '.', ',', [rfReplaceAll]);
    result := sysutils.StrToFloat(s);
  except
    result := 0.0;
  end;
  end;
end;

Function TColorToTVector3f(Color : TColor) : TVector3f;
begin

Result.X := GetRValue(Color) / 255;
Result.Y := GetGValue(Color) / 255;
Result.Z := GetBValue(Color) / 255;

end;

Function TVector3fToTColor(Vector3f : TVector3f) : TColor;
begin

Result := RGB(trunc(Vector3f.X*255),trunc(Vector3f.Y*255),trunc(Vector3f.Z*255));

end;

//Helios_Engine.Pas
Function DoesModFolderExist(ModName : string) : Boolean;
begin
Result := fileexists(GAME_FOLDER + '\' + ModName + '\liblist.gam');
end;


Procedure ChangeModFolder(ModName : string);
begin
  if DoesModFolderExist(ModName) then
  MOD_FOLDER := ModName;

  BumpVP := ARB_LoadShader(GL_VERTEX_PROGRAM_ARB, DoesFileExist('\shader\bump_vp.txt'));
  BumpFP := ARB_LoadShader(GL_FRAGMENT_PROGRAM_ARB, DoesFileExist('\shader\bump_fp.txt'));
end;

Procedure InitiateHelios(GameFolder,ModFolder, Params : String);
begin
GAME_FOLDER := GameFolder;
MOD_FOLDER := ModFolder;
gTimer := THiResTimer.Create;

BumpVP := ARB_LoadShader(GL_VERTEX_PROGRAM_ARB, DoesFileExist('\shader\bump_vp.txt'));
BumpFP := ARB_LoadShader(GL_FRAGMENT_PROGRAM_ARB, DoesFileExist('\shader\bump_fp.txt'));
end;

procedure glResizeWnd(Width, Height : Integer);
begin
SCREEN_WIDTH := Width;
SCREEN_HEIGHT := Height;

  if (Height = 0) then                // prevent divide by zero exception
    Height := 1;
  glViewport(0, 0, Width, Height);    // Set the viewport for the OpenGL window
  glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
  glLoadIdentity();                   // Reset View
//  gluPerspective(FOV, Width/Height, 1.0, DEPTH_OF_VIEW);  // Do the perspective calculations. Last value = max clipping depth
  gluPerspective(FOV, Width/Height, 4.0, DEPTH_OF_VIEW);  // Do the perspective calculations. Last value = max clipping depth
//  PerspectiveGL(FOV, Width/Height, 4.0, DEPTH_OF_VIEW);
//   glFrustum(-40.0, 40.0, -20.704225, 20.704225, 4.0, 4096.0);  //GLQuake Mode
  glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
  glLoadIdentity();                   // Reset View
end;

//Helios_FileSystem.Pas
Function DoesFileExist(Filename : String) : String; // If Exists filename is given as result
begin

if MOD_FOLDER <> '' then
if fileexists(GAME_FOLDER+'\'+MOD_FOLDER+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+MOD_FOLDER+'\'+Filename;
Exit;
end;

if fileexists(GAME_FOLDER+'\'+HELIOS_MODFOLDER+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+HELIOS_MODFOLDER+'\'+Filename;
Exit;
end;

If Basedir <> '' then
if fileexists(GAME_FOLDER+'\'+Basedir+'\'+Filename) then
begin
Result := GAME_FOLDER+'\'+Basedir+'\'+Filename;
Exit;
end;

if fileexists(Filename) then
begin
Result := Filename;
Exit;
end;

Result := '';

end;

//Q3BSP.Pas
procedure SetDefaults;
begin
  //glDepthFunc(GL_LEQUAL);  //
  //glEnable(GL_DEPTH_TEST); //
  glDisable(GL_ALPHA_TEST);
  glDepthMask(true);
  glDisable(GL_BLEND);
  {glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);}

  glDisable(GL_COLOR);  //
{  if CG_MultiTexture then begin //
    glActiveTexture(GL_TEXTURE1);
    glDisableClientState(GL_COLOR_ARRAY);
    glActiveTexture(GL_TEXTURE0);
  end;   }
  glDisableClientState(GL_COLOR_ARRAY);   //
  glColor3ub(255,255,255);  //
  glCullFace(GL_FRONT);
  //glEnable(GL_CULL_FACE);  //
end;

Procedure IsGame; // Dummy, used for compatibility with Helios
begin
    //
end;

Procedure DrawGfxToScreen(Texture : Cardinal; X,Y,Width,Height,Alpha : Single; Colour : TVector3f);
begin

glLoadIdentity();                                       // Reset The View

    glDisable(GL_CULL_FACE);

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glMatrixMode(GL_PROJECTION);
    glPushMatrix;
    glLoadIdentity;

    glDisable(GL_DEPTH_TEST);                                                     // Disables Depth Testing

    glOrtho(0, SCREEN_WIDTH, 0, SCREEN_HEIGHT, -1, 1);                                    // Set Up An Ortho Screen
    glMatrixMode(GL_MODELVIEW);                                                   // Select The Modelview Matrix
    glPushMatrix();
    glLoadIdentity();

    //RenderBackground;

    glColor4f(Colour.X, Colour.Y, Colour.Z, Alpha);

    //TEXTURE LAYER 0
    glActiveTexture(GL_TEXTURE0);
    GLBindTexture(GL_TEXTURE_2D, Texture);
    LastTexID := Texture;

    glEnable(GL_TEXTURE_2D);
    //TEXTURE LAYER 1
    glActiveTexture(GL_TEXTURE1);
    glDisable(GL_TEXTURE_2D);

    glBegin(GL_QUADS);
      //1
    glMultiTexCoord2f{ARB}(GL_TEXTURE0, 0, 0);
    glVertex2f(X+0, Y+0);

      //2
    glMultiTexCoord2f{ARB}(GL_TEXTURE0, 1, 0);
    glVertex2f(X+WIDTH, Y+0);

      //3
    glMultiTexCoord2f{ARB}(GL_TEXTURE0, 1, 1);
    glVertex2f(X+WIDTH, Y+HEIGHT);

      //4
    glMultiTexCoord2f{ARB}(GL_TEXTURE0, 0, 1);
    glVertex2f(X+0, Y+HEIGHT);

    glEnd;

    glPopMatrix;
    glMatrixMode(GL_PROJECTION);
    glPopMatrix;
    glPopAttrib;
    glMatrixMode(GL_MODELVIEW);
    glEnable(GL_CULL_FACE);
    glDisable(GL_BLEND);
end;

begin

FakeLightmap := TColorToTVector3f(RGB(255,255,255));
FakeLightmapB := False;

end.
